Type: Package
Package: simstudy
Title: Simulation of Study Data
Version: 0.9.2
Date: 2026-02-06
Authors@R: 
    c(person(given = "Keith",
             family = "Goldfeld",
             role = c("aut", "cre"),
             email = "keith.goldfeld@nyulangone.org",
             comment = c(ORCID = "0000-0002-0292-8780")),
      person(given = "Jacob",
             family = "Wujciak-Jens",
             role = "aut",
             email = "jacob@wujciak.de",
             comment = c(ORCID = "0000-0002-7281-3989")))
Description: Simulates data sets in order to explore modeling
    techniques or better understand data generating processes. The user
    specifies a set of relationships between covariates, and generates
    data based on these specifications. The final data sets can represent
    data from randomized control trials, repeated measure (longitudinal)
    designs, and cluster randomized trials. Missingness can be generated
    using various mechanisms (MCAR, MAR, NMAR).
License: GPL-3
URL: https://github.com/kgoldfeld/simstudy,
        https://kgoldfeld.github.io/simstudy/,
        https://kgoldfeld.github.io/simstudy/dev/
BugReports: https://github.com/kgoldfeld/simstudy/issues
Depends: R (>= 4.1.0)
Imports: data.table, glue, methods, mvnfast, Rcpp, backports, fastglm,
        pbv (>= 0.5.47)
Suggests: covr, dplyr, formatR, gee, ggplot2, grid, gridExtra,
        hedgehog, knitr, magrittr, Matrix, mgcv, ordinal, pracma,
        rmarkdown, scales, splines, survival, testthat, gtsummary,
        broom.helpers, survminer, katex, dirmult, rms, lmerTest
LinkingTo: Rcpp, pbv (>= 0.5.47), fastglm
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-02-08 22:02:04 UTC; KSG
Author: Keith Goldfeld [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0292-8780>),
  Jacob Wujciak-Jens [aut] (ORCID:
    <https://orcid.org/0000-0002-7281-3989>)
Maintainer: Keith Goldfeld <keith.goldfeld@nyulangone.org>
Repository: CRAN
Date/Publication: 2026-02-09 08:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 09:38:24 UTC; unix
Archs: simstudy.so.dSYM
