# Generated by roxygen2: do not edit by hand

export(addColumns)
export(addCompRisk)
export(addCondition)
export(addCorData)
export(addCorFlex)
export(addCorGen)
export(addDataDensity)
export(addMarkov)
export(addMultiFac)
export(addPeriods)
export(addSynthetic)
export(betaGetShapes)
export(blockDecayMat)
export(blockExchangeMat)
export(catProbs)
export(defCondition)
export(defData)
export(defDataAdd)
export(defMiss)
export(defRead)
export(defReadAdd)
export(defReadCond)
export(defRepeat)
export(defRepeatAdd)
export(defSurv)
export(delColumns)
export(gammaGetShapeRate)
export(genCatFormula)
export(genCluster)
export(genCorData)
export(genCorFlex)
export(genCorGen)
export(genCorMat)
export(genCorOrdCat)
export(genData)
export(genDataDensity)
export(genDummy)
export(genFactor)
export(genFormula)
export(genMarkov)
export(genMiss)
export(genMixFormula)
export(genMultiFac)
export(genNthEvent)
export(genObs)
export(genOrdCat)
export(genSpline)
export(genSurv)
export(genSynthetic)
export(grouped)
export(iccRE)
export(logisticCoefs)
export(mergeData)
export(negbinomGetSizeProb)
export(scenario_list)
export(survGetParams)
export(survParamPlot)
export(trimData)
export(trtAssign)
export(trtObserve)
export(trtStepWedge)
export(updateDef)
export(updateDefAdd)
export(viewBasis)
export(viewSplines)
import(Rcpp)
import(data.table)
import(glue)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(fastglm,fastglm)
importFrom(methods,is)
importFrom(pbv,pbvnorm)
useDynLib(simstudy)
useDynLib(simstudy, .registration = TRUE)
